# VW Fix Vectorscript Constant Names v1.0 230610
# Usage
# Convert constant name k<Uppercase char><name remainder> to k_<Uppercase All chars>
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk

import re
from Npp import editor, notepad
from os import path
import time

constantnames = {}
startset = False
unreferencedinfile = False
editunreferenced = False
unreferencedcount = 0

# ============================================================processing functions
def checkconstant():
	pat = '^Const' #any precompiled command
	matches = []
	editor.research(pat, lambda m: matches.append(m.span()), re.IGNORECASE)
	return matches == []

def processmulticomment(teststr):
	global startset

	pat = re.compile("[\{\(]\*")
	starttest = bool(re.search(pat,teststr))

	pat = re.compile("\*[\}\)]")
	endtest = bool(re.search(pat,teststr))
		
	if starttest: #start of multi comment block
		startset = True
	endset = (endtest and startset) or (starttest and endtest) #got multi comment block
	
	return endset
	
def processinclude(teststr,status):

	def getfilepath(tempstr):
		spaceref = tempstr.find(' ')

		fpath = tempstr[spaceref:-1]
		return fpath
	
	pat = re.compile("\{\$\S.*\}") #any precompiled command
	btest = re.search(pat,teststr)
	if btest and status:
		filepath = getfilepath(teststr)
		fexists = path.isfile(filepath.strip())
		print('Constant references in:')
		print(teststr)
		if not fexists:
			print("File not found")
		print('')

	return bool(btest)

def processsinglecomment(teststr):
	tempstr = teststr.strip()
	btest = tempstr.startswith("{") and tempstr.endswith("}")
	
	return bool(btest)

def processcommas(teststr):
	if teststr.find(",") >= 0:
		result = re.split(",", teststr)
		# Process commas
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += ", " + result[val].strip()
	# else:
		# tempstr = teststr

	return teststr

def processbrackets(teststr):
	leftbracket = teststr.find("(")
	rightbracket = teststr.find(")")
	
	if leftbracket >= 0:
		result = re.split("\(", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += "( " + result[val].strip()

	if rightbracket >= 0:
		result = re.split("\)", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += " )" + result[val].rstrip()

	return teststr

def processconstants(teststr,lineNumber):
	global searchnames, unreferencedinfile, unreferencedcount, errcnt

	tempstr = teststr.strip() # strip leading trailng space

	# Old way
	# #Check comment
	# fno = tempstr.find("{") # test left curly bracket
	# if fno >= 0:
		# tempstr = tempstr.split("{")[0]
	
	# pat = re.compile("\'\S.*\'") # test single quote enclosure
	# btest = re.findall(pat,tempstr)
	# if btest:
		# for i in btest:
			# tempstr = tempstr.replace(i,'')

	# pat = re.compile("\"\S.*\"") # test double quote enclosure
	# btest = re.findall(pat,tempstr)
	# if btest:
		# for i in btest:
			# tempstr = tempstr.replace(i,'')

	# tempstr = tempstr.replace(';','')  # test semi colon
	# tempstr = tempstr.replace('(',' ') # test left bracket
	# tempstr = tempstr.split(')')       # test right bracket
	# tempstr = ' '.join(tempstr)

	# tempstr = tempstr.split(',') # split by comma
	# tempstr = ' '.join(tempstr)

	# tempstr = tempstr.split(':=') # split by assign
	# tempstr = ' '.join(tempstr)
	
	# tempstr = tempstr.split(':') # split by case option
	# tempstr = ' '.join(tempstr)

	# tempstr = tempstr.split('..') # split by array definition
	# tempstr = ' '.join(tempstr)

	# tempstr = tempstr.split('-') # split by minus sign
	# tempstr = ' '.join(tempstr)

	# tempstr = tempstr.replace('[',' ') # test left square bracket
	# tempstr = tempstr.split(']') # split by case option
	# tempstr = ' '.join(tempstr)
	
	# Alternative
	tempstr = re.sub(r'\W+', ' ', tempstr).strip()
	
	unreferenced = False

	foundwords = tempstr.split(' ') # split by space check TAB
	for word in foundwords:
		if word != '':
			tempword = word.lower()
			if tempword in constantnames:
				constant = constantnames[tempword]
				if constant and constant != word:
					teststr = teststr.replace(word,constant)

					if teststr.find(constant) == -1:
						print(teststr)
						print('===============================================================================Error in constant replacement')
						errcnt += 1

			else: #check for unreferenced constants

				# pat = re.compile(r'\bk[A-Z0-9_].*')
				pat = r"\bk[A-Z][^,\s]+"
				btest = re.findall(pat,word)
				
				if btest != []:
					print(str(lineNumber+1)+': Unreferenced '+btest[0])
					editor.markerAdd(lineNumber,20)

					if editunreferenced:
						unreferencedcount += 1
						constname = btest[0].strip()

						if constname.startswith('k_'):
							newconstname = 'k_' + constname.upper()[2:]
						elif constname.startswith('k'):
							newconstname = 'k_' + constname.upper()[1:]
						else:
							newconstname = 'k_' + constname.upper()

						oldstr = teststr
						teststr = teststr.replace(word,newconstname)

						if teststr.find(newconstname) == -1:
							print(teststr)
							print('===============================================================================Error in constant replacement')
							errcnt += 1

					else:
						unreferenced = True
	
	if unreferenced:
		unreferencedinfile = True

	return teststr

# =====================================================Routines for initial scan for constants
def getstatusstart(teststr):
	teststr = teststr.lower()
	return teststr.startswith('const')

def getstatusend(teststr,status):
	endset = False
	if status:
		teststr = teststr.lower()
		for item in ['var','type','procedure','function']:
			if teststr.startswith(item):
				endset = True
	return endset

def getconstantname(teststr):
	constname = ''
	pat = re.compile(r"\b\S*\s*=.*;")
	btest = re.findall(pat,teststr)
	if btest != []:
		constname = btest[0].split()[0]
	return constname

# ============================================================end of processing functions

def getConstants(contents, lineNumber, totalLines):
	global constantnames, status
	
	tempstr = contents.strip()
	
	# check for $INCLUDE and single line comments
	if tempstr != '':
		if not (processsinglecomment(tempstr) and processinclude(tempstr,status)):
			if not status and getstatusstart(tempstr):
				status = True
			elif status and getstatusend(tempstr,status):
				status = False
			elif status:
				constname = getconstantname(tempstr)
				if constname != '' and not (constname in constantnames):
					if constname.startswith('k_'):
						newconstname = 'k_' + constname.upper()[2:]
					elif constname.startswith('k'):
						newconstname = 'k_' + constname.upper()[1:]
					else:
						newconstname = 'k_' + constname.upper()
					constantnames[constname.lower()] = newconstname

def testContents(contents, lineNumber, totalLines):
	global constantnames
	global startset

	teststr = contents
	endset = processmulticomment(teststr)

	if endset:
		startset = False #reset for next call
		
	elif not startset:
		if not processinclude(teststr,False): # if include found leave
			if not processsinglecomment(teststr): # if single comment found leave
					teststr = processbrackets(teststr)
					teststr = processcommas(teststr)
					if teststr:
						teststr = processconstants(teststr,lineNumber)
					if contents != teststr:
						editor.replaceLine(lineNumber,teststr.rstrip())

# ======================================Main
console.show()
console.clear()

print('VW Fix Vectorscript Constant Names v1.0 230610')
fname = notepad.getCurrentFilename()

import os
name, extension = os.path.splitext(fname)

if extension in ['.vss','.px']:

	result = notepad.messageBox('Edit constants not in CONST section?', 'Edit unreferenced constants',MESSAGEBOXFLAGS.YESNOCANCEL)
	# MESSAGEBOXFLAGS.YESNOCANCEL  Yes = 6 No = 7 Cancel = 2
		
	if result != 2:
		editunreferenced = result
		
		print('Processing STARTED')
		print('')
		starttime = time.time()
		editor.beginUndoAction()
		editor.markerDeleteAll(20)
		editor.indicSetStyle(20, 8)
		editor.indicSetFore(20, (192, 0, 0))
		editor.indicSetAlpha(20, 50)
		
		errcnt = 0
		status = checkconstant()

		# check for no Const for include files
		editor.forEachLine(getConstants) # get constant definitions
		editor.forEachLine(testContents) # change constant names
		
		if unreferencedinfile:
			result = notepad.messageBox('Warning: Constants not defined in file CONST section(s)', 'Unreferenced Constants',MESSAGEBOXFLAGS.ICONINFORMATION)
		elif unreferencedcount > 0:
			print('')
			print('Edited unreferenced Constants count = '+str(unreferencedcount))

		editor.endUndoAction()
		endtime = time.time()
		print('')
		if errcnt > 0: print('{} error(s) occured in replacement'.format(errcnt))
		print('Processing ENDED ('+str(endtime-starttime)+' seconds)')

else:
	print(fname)
	print('Current file not a Vectorscript file - aborting')
	result = notepad.messageBox('Current file not a Vectorscript file', 'Invalid File',MESSAGEBOXFLAGS.ICONINFORMATION)

# ======================================Main