# Fix Vectorscript Global Variable Names v1.0 230610
# Usage
# Convert global variabel name g<Uppercase char><remainder chars> to g_<Uppercase char><remainder chars>
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk

import re
from Npp import editor, notepad
# editor, editor1, editor2, notepad, console

varnames = {}
status = False
endvarset = False
startset = False
unreferencedinfile = False
editunreferenced = False
unreferencedcount = 0
referencedcount = 0

# ============================================================processing functions
def processmulticomment(teststr):
	global startset

	pat = re.compile("[\{\(]\*")
	starttest = bool(re.search(pat,teststr))

	pat = re.compile("\*[\}\)]")
	endtest = bool(re.search(pat,teststr))
		
	if starttest: #start of multi comment block
		startset = True
	endset = (endtest and startset) or (starttest and endtest) #got multi comment block
	
	return endset
	
def processinclude(teststr,status):	
	pat = re.compile("\{\$\S.*\}") #any precompiled command
	btest = re.search(pat,teststr)

	return bool(btest)

def processsinglecomment(teststr):
	tempstr = teststr.strip()
	btest = tempstr.startswith("{") and tempstr.endswith("}")
	
	return bool(btest)

def processcommas(teststr):
	if teststr.find(",") >= 0:
		result = re.split(",", teststr)
		# Process commas
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += ", " + result[val].strip()

	return teststr

def processbrackets(teststr):
	leftbracket = teststr.find("(")
	rightbracket = teststr.find(")")
	
	if leftbracket >= 0:
		result = re.split("\(", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += "( " + result[val].strip()

	if rightbracket >= 0:
		result = re.split("\)", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += " )" + result[val].rstrip()

	return teststr

def processvariables(teststr,lineNumber):
	global searchnames, unreferencedinfile, unreferencedcount, referencedcount, errcnt

	tempstr = teststr.strip() # strip leading trailng space

	# Old way
	# #Check comment
	# fno = tempstr.find("{") # test left curly bracket
	# if fno >= 0:
		# tempstr = tempstr.split("{")[0]
	
	# pat = re.compile("\'\S.*\'") # test single quote enclosure
	# btest = re.findall(pat,tempstr)
	# if btest:
		# for i in btest:
			# tempstr = tempstr.replace(i,'')

	# pat = re.compile("\"\S.*\"") # test double quote enclosure
	# btest = re.findall(pat,tempstr)
	# if btest:
		# for i in btest:
			# tempstr = tempstr.replace(i,'')

	# tempstr = tempstr.replace(';','')  # test semi colon
	# tempstr = tempstr.replace('(',' ') # test left bracket

	# tempstr = tempstr.replace('[',' ') # test left square bracket

	# for i in [')',',',':','+','-','*','/','^','=',']','.','>','<',' ']:
		# tempstr = tempstr.split(i)
		# tempstr = ' '.join(tempstr)

	# Alternative
	tempstr = re.sub(r'\W+', ' ', tempstr).strip()
	
	unreferenced = False

	foundwords = tempstr.split(' ') # split by space check TAB
	for word in foundwords:
		if word != '':
			tempword = word.lower()
			if tempword in varnames:
				varname = varnames[tempword]

				if varname and varname != word:
					referencedcount += 1
					teststr = teststr.replace(word,varname)
					
					if teststr.find(varname) == -1:
						print(teststr)
						print('===============================================================================Error in global variable replacement')
						errcnt += 1

			else: #check for unreferenced variables
				pat = re.compile(r'\bg[A-Z0-9_].*')
				btest = re.findall(pat,word)
				if btest != [] and btest[0] != word: #KEEP AN EYE ON
					print(str(lineNumber+1)+': Unreferenced '+btest[0])
					editor.markerAdd(lineNumber,20)

					if editunreferenced:
						unreferencedcount += 1
						varname = btest[0]
						print((varname,word))
						if varname.startswith('g_'):
							newvarname = 'g_' + varname[2:].title()
						elif varname.startswith('g'):
							newvarname = 'g_' + varname[1:].title()
						else:
							newvarname = 'g_' + varname.title()

						teststr = teststr.replace(word,newvarname)
						if teststr.find(newvarname) == -1:
							print(teststr)
							print('===============================================================================Error in global variable replacement')
							errcnt += 1
					else:
						unreferenced = True
	
	if unreferenced:
		unreferencedinfile = True

	return teststr

# =====================================================Routines for initial scan for variables
def getstatusstart(teststr):
	teststr = teststr.lower()
	return teststr.startswith('var')

def getstatusend(teststr,status):
	endset = False
	if status:
		teststr = teststr.lower()
		for item in ['procedure','function']:
			if teststr.startswith(item):
				endset = True
	return endset

def getvarnames(teststr):
	varnamelist = []
	if teststr.find(':') != -1:
		pat = r"g[A-Z][^,\s]+" #get comma seperated list of variables starting with g<Uppercase letter>
		varnamelist = re.findall(pat,teststr)
	return varnamelist

# ============================================================end of processing functions

def getVariables(contents, lineNumber, totalLines):
	global varnames, status, endvarset
	
	tempstr = contents.strip()
	
	# check for $INCLUDE and single line comments
	if tempstr != '':
		if not (processsinglecomment(tempstr) and processinclude(tempstr,status)):
			if not endvarset and getstatusstart(tempstr):
				status = True
			elif status and getstatusend(tempstr,status):
				status = False
				endvarset = True
			elif status:
				varnamelist = getvarnames(tempstr)
				if varnamelist != []:
					for varname in varnamelist:
						if not (varname in varnames):
							if varname.startswith('g_'):
								newvarname = 'g_' + varname[2:]
							elif varname.startswith('g'):
								newvarname = 'g_' + varname[1:]
							else:
								newvarname = 'g_' + varname
							varnames[varname.lower()] = newvarname

def testContents(contents, lineNumber, totalLines):
	global varnames
	global startset

	teststr = contents
	endset = processmulticomment(teststr)

	if endset:
		startset = False #reset for next call
		
	elif not startset:
		if not processinclude(teststr,False): # if include found leave
			if not processsinglecomment(teststr): # if single comment found leave
					teststr = processbrackets(teststr)
					teststr = processcommas(teststr)
					if teststr:
						teststr = processvariables(teststr,lineNumber)
					if contents != teststr:
						editor.replaceLine(lineNumber,teststr.rstrip())

# ======================================Main
console.show()
console.clear()

import time

print('Fix Vectorscript Global Variable Names v1.0 230610')
fname = notepad.getCurrentFilename()

import os
name, extension = os.path.splitext(fname)

if extension in ['.vss','.px']:

	result = notepad.messageBox('Edit Global Variables not in VAR section?', 'Edit unreferenced Global Variables',MESSAGEBOXFLAGS.YESNOCANCEL)
	# MESSAGEBOXFLAGS.YESNOCANCEL  Yes = 6 No = 7 Cancel = 2
		
	if result != 2:
		editunreferenced = result
		
		print('Processing STARTED')
		print('')
		starttime = time.time()
		editor.beginUndoAction()
		editor.markerDeleteAll(20)
		editor.indicSetStyle(20, 8)
		editor.indicSetFore(20, (192, 0, 0))
		editor.indicSetAlpha(20, 50)
		
		errcnt = 0

		editor.forEachLine(getVariables) # get variable definitions
		editor.forEachLine(testContents) # change variable names

		print('Edited referenced Global Variable count = '+str(referencedcount))
		
		if unreferencedinfile:
			result = notepad.messageBox('Warning: Global Variables not defined in file VAR section(s)', 'Unreferenced Global Variables',MESSAGEBOXFLAGS.ICONINFORMATION)
		elif unreferencedcount > 0:
			print('')
			print('Edited unreferenced Global Variable count = '+str(unreferencedcount))

		editor.endUndoAction()
		endtime = time.time()
		print('')
		if errcnt > 0: print('{} error(s) occured in replacement'.format(errcnt))
		print('Processing ENDED ('+str(endtime-starttime)+' seconds)')

else:
	print(fname)
	print('Current file not a Vectorscript file - aborting')
	result = notepad.messageBox('Current file not a Vectorscript file', 'Invalid File',MESSAGEBOXFLAGS.ICONINFORMATION)

# ======================================Main