# Vectorscript Get Input Parameters v1.0 230629
# Usage
# From selected word or previous unselected word at cursor position
# Check if routine exists and append defined parameters within brackets
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk

import xml.etree.ElementTree as ET
import re
from Npp import editor, notepad, console
import os.path

def getparamstring(paramstr):
	tempstr = paramstr.split(')')[0]
	tempstr = tempstr.split(';')
	newparmstr = '( '
	varrefs = []

	for varitem in tempstr:
		item = varitem
		if item.find(':') != -1:
			item = item.split(':')
			varfound = item[0].find('VAR ') != -1
			varnames = item[0].split(',')

			if varfound:
				varnames[0] = varnames[0].replace('VAR ','').strip()
			vartype = item[1]

			if vartype.find('[') != -1: #arrays
				vartype = vartype.split('[')[0]

			for name in varnames:
				if varfound:
					name = 'VAR_' + name
				varrefs.append(name.strip() + '_' + vartype.strip())

	newparmstr += ', '.join(varrefs)
	newparmstr += ' )'

	return newparmstr

def getfileparamstring(linedef,callname):
	btest = []
	if linedef != '':
		ldeflower = linedef.lower()
		if ldeflower.startswith('procedure') | ldeflower.startswith('function'):
			callname = callname+'('
			if ldeflower.find(callname) != -1:
				btest = linedef
	return btest

# ===================================================main
console.clear()

seltext = editor.getSelText()
callname = ''

if len(seltext) > 0:
	curpos = editor.getSelectionEnd()
	editor.gotoPos(curpos)
	startbracketset = False
	endbracketset = False

	if seltext.find('(') != -1:
		seltext = seltext.split('(')[0]

	if seltext.find(' ') != -1:
		seltext = seltext.split(' ')[0]
	callname = seltext

if callname == '':
	curpos = editor.getCurrentPos()
	lastchar = editor.getCharAt(curpos-1)
	
	#check for a previous word

	startbracketset = (lastchar == 40) # left bracket
	if startbracketset:
		wordpos = curpos-1
	else:
		wordpos = curpos

	startpos = editor.wordStartPosition(wordpos, True)
	callname = editor.getTextRange(startpos, wordpos)

if startbracketset:
	nextchar = editor.getCharAt(curpos)
	endbracketset = (nextchar == 41) # right bracket
	
if callname == '':
	result = notepad.messageBox('No indicated routine preceeding the cursor', 'No Routine Found',MESSAGEBOXFLAGS.ICONINFORMATION)

else:
	npppath = notepad.getNppDir()
	xmlfile =  npppath + '\\autoCompletion\\pascal.xml'
	
	tree = ET.parse(xmlfile)
	root = tree.getroot()

	pat = ".//AutoComplete/KeyWord[@name='"+callname+"']"
	elem=root.findall(pat)

	found = elem != []
	if found:
		elem = elem[0]
		found = elem.getchildren() != []
		if found:
			overload = elem[0]
			descr = overload.attrib['descr']
			params = []
			for param in overload:
				name = param.attrib['name']
				name = name.split(':')
				if name[1].find('[') != -1: #handles arrays
					name[1] = name[1].split('[')[0]

				name[0] = name[0].strip()
				if name[0].find(' ') != -1:
					name[0] = name[0].replace(' ','_')

				tempstr = name[0] + '_' + name[1].strip()
				params.append(tempstr)

			paramstr = '( '+', '.join(params)+' )'

			editor.callTipShow(curpos, descr)

			if startbracketset and endbracketset:
				tempstr = paramstr[1:-1]
				editor.insertText(curpos, tempstr)
				curpos += len(tempstr)+1

			elif startbracketset:
				tempstr = paramstr[1:]
				editor.insertText(curpos, tempstr)
				curpos += len(tempstr)

			else:
				tempstr = paramstr
				editor.insertText(curpos, paramstr)
				curpos += len(tempstr)

	if not found: #check local routines
		matches = []
		paramstr = ''

		pat = r"(?i)^procedure +{}\((.*)\)".format(callname)
		editor.research(pat, lambda m: matches.append(m.span()))
		for (s,e) in matches:
			paramstr = editor.getTextRange(s, e)
			desc = paramstr

		if paramstr == '':
			matches = []
			pat = r"(?i)^function +{}\((.*)\).*".format(callname)
			editor.research(pat, lambda m: matches.append(m.span()))
			for (s,e) in matches:
				paramstr = editor.getTextRange(s, e)
			
			desc = paramstr
			rightbracket = paramstr.find(')')
			if rightbracket != -1: #strip return type
				paramstr = paramstr[0:rightbracket+1]

		found = (paramstr != '')
		if found:
			editor.callTipShow(curpos, desc)

			paramstr = paramstr.split('(')[1][:-1]
			paramstr = getparamstring(paramstr)
			editor.insertText(curpos, paramstr)
			curpos += len(paramstr)

	if not found: #check external routines
		matches = []
		pat = r"^(?i)\bVAR".format(callname)
		editor.research(pat, lambda m: matches.append(m.span()))
		if matches == []:
			startpos = 0
		else:
			startpos = matches[0][0]

		matches = []
		paramfound = False
		pat = "\{\$INCLUDE (.*)\}"

		editor.research(pat, lambda m: matches.append(m.span()))
		for (s,e) in matches:
			filestr = editor.getTextRange(s, e)
			filestr = filestr.replace('{$INCLUDE ','')[:-1]

			if s >= startpos and os.path.isfile(filestr):
				f = open(filestr)
				# fcontent = f.read()
				fcontent = f.readlines()
				f.close()
				
				for linedef in fcontent:
					paramstr = getfileparamstring(linedef,callname)
					found = (paramstr != [])
					if found:
						break
				if found:
					editor.callTipShow(curpos, paramstr)

					paramstr = paramstr.split('(')[1][:-1]
					paramstr = getparamstring(paramstr)
					editor.insertText(curpos, paramstr)
					curpos += len(paramstr)

			if found: break

	if not found:
		result = notepad.messageBox('No parameters found for indicated routine in searched locations', 'No Parameters Found',MESSAGEBOXFLAGS.ICONINFORMATION)

	editor.gotoPos(curpos)

	
