# Vectorscript Get Structure Type for selection v1.0 230630
# Usage
# Display list of inbuilt types and STRUCTURE types
# Select from list and insert at cursor position
# If cursor located at a period then check params of previous word as STRUCTURE type
# otherwise select a Structure type in current file or INCLUDE file
#
# Note. Only top level structures handled
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk
#
# Edited 230905

import re
from Npp import editor
import os.path

def getincluderefs(filestr):
	f = open(filestr)
	fcontent = f.readlines()
	f.close()
	paramlist = []
	vartypedict = {}
	structset = False
	pat = r"\b(.*)\s.*\:.*;" #Var line

	for linestr in fcontent:
		lowerline = linestr.lower()
		params = re.findall(pat,linestr) #get Var line params
		
		if lowerline.find('structure') != -1: # mitredef = STRUCTURE
			structname = linestr.split('=')[0].strip()
			structset = True

		elif params != []:
			paramstr = params[0].strip()
			if paramstr.find(',') != -1:
				paramlist += paramstr.split(',')
			else:
				paramlist.append(paramstr)

		elif lowerline.find('end;') != -1:
			if paramlist != []:
				vartypedict[structname] = paramlist
				paramlist = []
				structset = False

	return vartypedict

# ===================================================main
console.hide()
seltext = editor.getSelText().strip()
typename = '' #default empty

if len(seltext) > 0:
	result = notepad.messageBox("Deselect/Delete text and place cursor", 'Selection Error', MESSAGEBOXFLAGS.ICONINFORMATION)
	# TODO Possible use of selection
	# curpos = editor.getSelectionEnd()
	# dotset = seltext.endswith('.')
	# if dotset:
		# typename = seltext.rstrip('.')
	# editor.setEmptySelection(curpos)

else:
# No selection set
	curpos = editor.getCurrentPos()
	dotchar = editor.getTextRange(curpos-1,curpos)
	dotset = (dotchar == '.')
	if dotset:
		editor.wordLeftEnd() # skip dot
		editor.wordLeftEnd()
		startpos = editor.getCurrentPos()
		typename = editor.getTextRange(startpos,curpos-1)
# No selection set end

	matches = []

	#Get local types between TYPE and VAR of procedure / function
	pat = r"^(?i)\bTYPE"
	editor.research(pat, lambda m: matches.append(m.span()))
	if matches == []:
		startpos = 0
	else:
		startpos = matches[0][0]

	matches = []

	#check for VAR declaration
	pat = r"^(?i)\bVAR"
	editor.research(pat, lambda m: matches.append(m.span()))

	if matches != []:
		endpos = matches[0][0]

	else: #backstop check for procedure or function
		pat = r"(?i)procedure*"
		editor.research(pat, lambda m: matches.append(m.span()))
		if matches == []:
			pat = r"(?i)function*"
			editor.research(pat, lambda m: matches.append(m.span()))
			if matches != []:
				endpos = matches[0][0]
			else:
				endpos = 0 #no scan will be performed
		else:
			endpos = matches[0][0]

	startline = editor.lineFromPosition(startpos)+2
	endline   = editor.lineFromPosition(endpos)

	vartypes = {} #dictionary {structname, params}
	paramlist = []
	structset = False
	structname = ''

	for lineno in range(startline,endline):
		linestr = editor.getLine(lineno)
		if linestr != '':
			lowerline = linestr.lower()
			if lowerline.find('{$include') != -1:	#check for INCLUDE
				filestr = linestr.strip().replace('{$INCLUDE ','')[:-1]
				if os.path.isfile(filestr):
					vartyperefs = getincluderefs(filestr)
					if vartyperefs != []:
						vartypes.update(vartyperefs) # merge dictionaries
			
			elif lowerline.find('structure') != -1: # mitredef = STRUCTURE
				structname = linestr.split('=')[0].strip()
				structset = True

			elif linestr.find(':') != -1:
				paramstr = linestr.split(':')[0].strip()
				if paramstr.find(',') != -1:
					paramlist += paramstr.split(',')
				else:
					paramlist.append(paramstr)

			elif lowerline.find('end;') != -1:
				if structname != '':
					vartypes.update({structname: paramlist}) 
				paramlist = []
				structset = False

	editor.gotoPos(curpos)

	if vartypes != {}:
		if dotset:
			if typename in vartypes:
				editor.autoCShow(0,' '.join(vartypes[typename]))
			else:
				result = notepad.messageBox("Identified variable has no associated Structure Type", 'Error in File', MESSAGEBOXFLAGS.ICONINFORMATION)
		else:
			btest = editor.autoCShow(0,' '.join(vartypes.keys()))
	else:
		result = notepad.messageBox("No Structure Type's defined in current or external file(s)", 'Error in File', MESSAGEBOXFLAGS.ICONINFORMATION)
