# VW Map Reference Code v1.0 230627
# Usage
# Append external INCLUDE file references after run() command
# References include constants, types, procedures and functions
# Check console for file errors
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk

import os
import re
from Npp import editor, notepad, console
import time

def addline(textstr):
	editor.newLine()
	editor.addText(textstr)

def addconstants():
	global fcontent
	pat = r"\bk.*\=.*\;"
	linelist = re.findall(pat,fcontent)
	for linestr in linelist:
		addline(linestr.strip().split(' ')[0])
	return

def addtypes():
	global fcontent
	pat = r"(?i)\b.*\=.*STRUCTURE"
	linelist = re.findall(pat,fcontent)
	for linestr in linelist:
		addline(linestr.strip().split(' ')[0])
	return

def addroutines():
	global fcontent
	#Handle nested procedures and functions
	for linedef in fcontent:
		if linedef != '':
			ldeflower = linedef.lower()
			if ldeflower.startswith('procedure') | ldeflower.startswith('function'):
				if linedef.find(' ') != -1:
					linedef = linedef.split(' ')[1]

				if linedef.find('(') != -1:
					linedef = linedef.split('(')[0]

				elif linedef.find(';') != -1:
					linedef = linedef.split(';')[0]
				
				if linedef != '':
					addline(linedef)

def getkeywordlocation(pat):
	matches = []
	editor.research(pat, lambda m: matches.append(m.span()))
	if matches == []:
		location = False
	else:
		location = matches[0] #first start and end position (s,e)
	return location

def getstartroutine():
	pat = r"^(?i)^procedure.*\(.*\)" # test proc with variables
	endproc = getkeywordlocation(pat) #first start and end position (s,e)
	if not endproc: # test proc with no variables
		pat = r"^(?i)^procedure.*\;"
		endproc = getkeywordlocation(pat) #first start and end position (s,e)

	pat = r"^(?i)^function.*\(.*\)"
	endfunc = getkeywordlocation(pat) #first start and end position (s,e)

	if endproc and endfunc:
		start = endproc if endproc[0] < endfunc[0] else endfunc
	elif endproc:
		start = endproc
	elif endfunc:
		start = endfunc
	else:
		start = False

	return start

def checklimits(filestart,startloc,endloc):
	return (filestart >= startloc[0]) and (filestart < endloc[0])

#==============================================main
result = notepad.messageBox('Map code in current file - continue?', 'Map Vectorscript Reference Code',MESSAGEBOXFLAGS.OKCANCEL)
if result == 1:
	console.show()
	console.clear()

	print('VW Map Reference Code v1.0 230627')
	print('Processing STARTED')
	print('')

	starttime = time.time()
	editor.beginUndoAction()

	editor.markerDeleteAll(20)
	editor.indicSetStyle(20, 8)
	editor.indicSetFore(20, (192, 0, 0))
	editor.indicSetAlpha(20, 50)
		
	matches = []
	pat = r"^(?i)\brun\(.*\);" #must have a run statement
	editor.research(pat, lambda m: matches.append(m.span()))
	filesnotfound = 0
	norunfile = False

	if matches != []:
		startpos = matches[0][1]
		endpos = editor.getTextLength() #end of document

		if endpos > startpos: # delete tail end
			editor.deleteRange(startpos,endpos-startpos)

		editor.gotoPos(endpos)

		matches = []
		includefiles = []
		pat = "\{\$INCLUDE (.*)\}"
		editor.research(pat, lambda m: matches.append(m.span()))

		for (s,e) in matches:
			filestr = editor.getTextRange(s, e)
			filestr = filestr.replace('{$INCLUDE ','')[:-1]
			if os.path.isfile(filestr):
				includefiles.append((s,filestr)) #startpos and filename
			else:
				filesnotfound += 1
				curline = editor.lineFromPosition(s)
				editor.markerAdd(curline,20)

		filecount =  len(includefiles)
		if includefiles != []:

			pat = r"^\b(?i)Const"
			startconst = getkeywordlocation(pat)

			pat = r"^\b(?i)Type"
			starttype = getkeywordlocation(pat)

			pat = r"^\b(?i)Var"
			startvar = getkeywordlocation(pat)

			startroutine = getstartroutine()
			
			if startconst:
				for enddef in [starttype,startvar,startroutine]:
					if enddef != False:
						endconst = enddef
						break
			
			if starttype:
				for enddef in [startvar,startroutine]:
					if enddef != False:
						endtype = enddef
						break
			
			if startvar:
				if startroutine != False:
					endvar = enddef
				else:
					endvar = False

			addline('{*')
			addline('External References')
			addline('Note: Make sure nested routines are indented in source files')
			if filesnotfound > 0:
				editor.newLine()
				addline('{} file(s) not found and marked with bookmarks'.format(filesnotfound))
			editor.newLine()
			
			for fileref in includefiles:
				fcontent = []
				f = open(fileref[1])
				fcontent = f.read()
				f.close()

				addline('+File')
				addline(fileref[1])
				editor.newLine()

				if startconst and checklimits(fileref[0],startconst,endconst):
					addline('+Constants')
					addconstants()
					editor.newLine()
					nonroutine = True

				elif starttype and checklimits(fileref[0],starttype,endtype):
					addline('+Types')
					addtypes()
					editor.newLine()

				elif endvar and startroutine: #(startvar and checklimits(fileref[0],startvar,endvar))
					f = open(fileref[1])
					fcontent = f.readlines() # read as list
					f.close()

					addline('+Routines')
					addroutines()
					editor.newLine()

			addline('*}')
		else:
			print('Current file has no $INCLUDE references')
	else:
		print('Current file has no Run routine')
		norunfile = True

	editor.endUndoAction()
	endtime = time.time()

	if not norunfile:
		print('{} INCLUDE file(s) processed'.format(filecount))
		
	if filesnotfound > 0:
		print('{} file(s) not found and marked with bookmark(s)'.format(filesnotfound))
		result = notepad.messageBox('{} file(s) not found and marked with bookmark(s)'.format(filesnotfound), 'Invalid File Path', MESSAGEBOXFLAGS.ICONINFORMATION)
		
	print('')
	print('Processing ENDED ('+str(endtime-starttime)+' seconds)')
