# Vectorscript Prettify Vectorscript v1.0 230610
# Usage
# Checks case of inbuilt Vectorscript routines
# Keeps leading line indents but pads each element by a single space
#
# Coded by Bill Wood
# Contact via www.whwsolution.co.uk
# Use at own risk

import xml.dom.minidom
import re
from Npp import editor, notepad, console

searchnames = {}
startset = False

# ============================================================processing functions

def processmulticomment(teststr):
	global startset

	pat = re.compile("[\{\(]\*")
	starttest = bool(re.search(pat,teststr))

	pat = re.compile("\*[\}\)]")
	endtest = bool(re.search(pat,teststr))
		
	if starttest: #start of multi comment block
		startset = True
	endset = (endtest and startset) or (starttest and endtest) #got multi comment block
	
	return endset
	
def processsinglecomment(teststr):
	tempstr = teststr.strip()
	btest = tempstr.startswith("{") and tempstr.endswith("}")
	
	return bool(btest)
	
def processinclude(teststr):
	pat = re.compile("\{\$\S.*\}") #any precompiled command
	btest = re.search(pat,teststr)
	
	# check if comment on same line as command
	
	return bool(btest)

def processcommas(teststr):
	if teststr.find(",") >= 0:
		result = re.split(",", teststr)
		# Process commas
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += ", " + result[val].strip()

	return teststr

def processbrackets(teststr):
	leftbracket = teststr.find("(")
	rightbracket = teststr.find(")")
	
	if leftbracket >= 0:
		result = re.split("\(", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += "( " + result[val].strip()

	if rightbracket >= 0:
		result = re.split("\)", teststr)
		teststr = result[0].rstrip()
		for val in range(1,(len(result))):
			teststr += " )" + result[val].rstrip()

	return teststr

def processcommands(teststr):
	global searchnames
	
	def checksyntax(word):
		btest = False
		
		cno = word.find('[')
		if cno != -1:
			word = word.replace('[','[ ')
			btest = True
		
		if word.find(']') != -1:
			word = word.replace(']',' ]')
			btest = True

		if word.find('+') != -1:
			word = word.replace('+',' + ')
			btest = True

		if word.find('-') > 0:
			word = word.replace('-',' - ')
			btest = True

		if word.find('/') != -1:
			word = word.replace('/',' / ')
			btest = True

		if word.find('*') != -1:
			word = word.replace('*',' * ')
			btest = True

		if word.find('<>') != -1:
			word = word.replace('<>',' <> ')
			btest = True
		elif word.find('>=') != -1:
			word = word.replace('>=',' >= ')
			btest = True
		elif word.find('<=') != -1:
			word = word.replace('<=',' <= ')
			btest = True
		else:	
			if word.find('>') != -1:
				word = word.replace('>','> ')
				btest = True

			elif word.find('<') != -1:
				word = word.replace('<',' <')
				btest = True

			elif word.find('=') != -1:
				word = word.replace('=',' = ')
				btest = True
		
		if word.find(':') != -1:
			word = word.replace(':',' : ')
			btest = True

		return btest,word	

	def stripmultispace(oldstr):
		pat = r'((^\s+)(.*))'
		m = re.findall(pat,oldstr)
		if m != []:
			lead = m[0][1]
			tail = m[0][2]
		else:
			lead = ''
			tail = oldstr
		
		tail = tail.split(' ')

		newtail = []
		for i in tail:
			if i != '':
				newtail.append(i)

		tail = ' '.join(newtail) #possible to use TAB instead

		return (lead + tail)

	tempstr = teststr.strip() # strip leading traing space

	#Check comment
	fno = tempstr.find("{") # test left curly bracket
	if fno >= 0:
		tempstr = tempstr.split("{")[0]
	
	pat = re.compile("\'\S.*\'") # test single quote enclosure
	btest = re.findall(pat,tempstr)
	if btest:
		for i in btest:
			tempstr = tempstr.replace(i,'')

	pat = re.compile("\"\S.*\"") # test double quote enclosure
	btest = re.findall(pat,tempstr)
	if btest:
		for i in btest:
			tempstr = tempstr.replace(i,'')

	tempstr = tempstr.replace(';','')  # test semi colon
	tempstr = tempstr.replace('(',' ') # test left bracket

	tempstr = tempstr.split(')')       # test right bracket
	tempstr = ' '.join(tempstr)

	# comma occurs in array[1,2]
	tempstr = tempstr.split(',') # split by comma
	tempstr = ' '.join(tempstr)

	tempstr = tempstr.split(':=') # split by assign
	tempstr = ' '.join(tempstr)

	foundwords = tempstr.split(' ') # split by space check TAB

	for word in foundwords:
		if word != '':
			oldword = word
			btest,word = checksyntax(word)
			if btest:
				teststr = teststr.replace(oldword,word)
			else:
				tempword = word.lower()
				if tempword in searchnames:
					command = searchnames[tempword]
					if command and command != word:
						teststr = teststr.replace(word,command)
			
			# start temporary fix till alternative found
			if teststr.find(' if(') != -1:
				teststr = teststr.replace(' if(',' if (')
			if teststr.find('while(') != -1:
				teststr = teststr.replace('while(','while (')
			if teststr.find(' to(') != -1:
				teststr = teststr.replace(' to(',' to (')
			if teststr.find(' downto(') != -1:
				teststr = teststr.replace(' downto(',' downto (')
			if teststr.find(':=(') != -1:
				teststr = teststr.replace(':=(',':= (')
			if teststr.find(': =') != -1:
				teststr = teststr.replace(': =',':=')
			if teststr.find(': ') != -1:
				teststr = teststr.replace(': ',' : ')
			if teststr.find('<(') != -1:
				teststr = teststr.replace('<(','< (')
			if teststr.find('>(') != -1:
				teststr = teststr.replace('>(','> (')
			if teststr.find('&(') != -1:
				teststr = teststr.replace('&(','& (')
			if teststr.find('|(') != -1:
				teststr = teststr.replace('|(','| (')
			if teststr.find('< =') != -1:
				teststr = teststr.replace('< =','<=')
			if teststr.find('> =') != -1:
				teststr = teststr.replace('> =','>=')
			if teststr.find(' and(') != -1:
				teststr = teststr.replace(' and(','  and (')
			if teststr.find(' or(') != -1:
				teststr = teststr.replace(' or(',' or (')
			# end temporary fix till alternative found
			
			teststr = stripmultispace(teststr)
				

	return teststr
	
# ============================================================end of processing functions

def initprocess():
	global searchnames
	npppath = notepad.getNppDir()
	xmlfile =  npppath + '\\autoCompletion\\pascal.xml'

	xml_doc = xml.dom.minidom.parse(xmlfile)
	root = xml_doc.documentElement
	keywords = xml_doc.getElementsByTagName('KeyWord')

	for kword in keywords:
		name = kword.getAttribute('name')
		searchnames[name.lower()] = name

def testContents(contents, lineNumber, totalLines):
	global startset

	teststr = contents
	endset = processmulticomment(teststr)

	if endset:
		startset = False #reset for next call
		
	elif not startset:
		if not processinclude(teststr): # if include found leave
			if not processsinglecomment(teststr): # if single comment found leave
					teststr = processbrackets(teststr)
					teststr = processcommas(teststr)
					if teststr:
						teststr = processcommands(teststr)
					if contents != teststr:
						editor.replaceLine(lineNumber,teststr.rstrip())
						
# ======================================Main
import time

result = notepad.messageBox('Pretiffy current file - continue?', 'Pretiffy Vectorscript',MESSAGEBOXFLAGS.OKCANCEL)
if result == 1:
	console.show()
	console.clear()

	print('Prettify Vectorscript v1.0 230610')
	fname = notepad.getCurrentFilename()

	import os
	name, extension = os.path.splitext(fname)

	if extension in ['.vss','.px']:
		print('Processing STARTED')
		starttime = time.time()
		editor.beginUndoAction()

		initprocess()
		editor.forEachLine(testContents)

		editor.endUndoAction()
		endtime = time.time()
		print('Processing ENDED ('+str(endtime-starttime)+' seconds)')

	else:
		print(fname)
		print('Current file not a Vectorscript file - aborting')
		result = notepad.messageBox('Current file not a Vectorscript file', 'Invalid File',MESSAGEBOXFLAGS.ICONINFORMATION)
		# MESSAGEBOXFLAGS.OK  OK = 1
		# MESSAGEBOXFLAGS.OKCANCEL  OK = 1 Cancel = 2
		# MESSAGEBOXFLAGS.YESNOCANCEL  Yes = 6 No = 7 Cancel = 2
		# MESSAGEBOXFLAGS.CANCELTRYCONTINUE  Cancel = 2 Try Again = 10 Continue = 11
		# MESSAGEBOXFLAGS.RETRYCANCEL  Retry = 4 Cancel = 2
		# MESSAGEBOXFLAGS.ICONINFORMATION  OK = 1
		# MESSAGEBOXFLAGS.ABORTRETRYIGNORE  Abort = 3 Retry = 4 Ignore = 5

# ======================================Main